-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000-2014                     --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@unican.es                     --
--          Jose Javier Gutierrez  gutierjj@unican.es                --
--          Jose Carlos Palencia   palencij@unican.es                --
--          Jose Maria Drake       drakej@unican.es                  --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------
with File_Execution; use File_Execution;
with Ada.Text_IO; use Ada.Text_IO;

procedure Test_File_Execution is
   No_Arguments : String_List(1..0);
   Arguments_To_sh : String_List(1..1):=
     String_List'(1=>new String'("/home/mgh/bin/gmast"));
   Success : Boolean;

   procedure Put_Success(Success : Boolean) is
   begin
      if Success then
         Put_Line("Success");
      else
         Put_Line("No Success");
      end if;
   end Put_Success;
begin
   -- Try a program that does not exist
   Put_Line("Try a program that does not exist");
   begin
      Execute_Command("gmast_",No_Arguments, Success);
      Put_Success(Success);
   exception
   when Program_Not_Found =>
      Put_Line("Error Message: Program not found");
   end;

   -- try a program that exists, with no arguments
   Put_Line("Try a program that exists, with no arguments");
   Execute_Command("gmast_analysis",No_Arguments, Success);
   Put_Success(Success);

   -- try a program that exists
   Put_Line("Try a program that exists, with one argument");
   Execute_Command("sh",Arguments_To_Sh, Success);
   Put_Success(Success);

   -- try a file that does not exist
   Put_Line("Try a file that does not exist");
   begin
      Execute_File("gmast_", Success);
      Put_Success(Success);
   exception
   when Program_Not_Found =>
      Put_Line("Error Message: File not found");
   end;

   -- try a file that exists
   Put_Line("Try a file that exists: mast_command");
   Execute_File("mast_command",Success);
   Put_Success(Success);

   -- try a file that exists
   Put_Line("Try a file that exists: mast_results_command");
   Execute_File("mast_results_command",Success);
   Put_Success(Success);

exception
   when Program_Not_Found =>
      Put_Line("Error: Program not found");
end Test_File_Execution;
